//	Библиотека для работы с Trema I2C расширителями выводов: http://iarduino.ru/shop/Expansion-payments/rasshiritel-na-8-vhodov-vyhodov-trema-modul.html
//  Версия: 1.0.0
//  Последнюю версию библиотеки Вы можете скачать по ссылке: http://iarduino.ru/file/285.html
//  Подробное описание функции бибилиотеки доступно по ссылке: http://wiki.iarduino.ru/page/rasshiritel-na-8-vhodov-vyhodov-trema-modul/
//  Библиотека является собственностью интернет магазина iarduino.ru и может свободно использоваться и распространяться!
//  При публикации устройств или скетчей с использованием данной библиотеки, как целиком, так и её частей,
//  в том числе и в некоммерческих целях, просим Вас опубликовать ссылку: http://iarduino.ru
//  Автор библиотеки: Панькин Павел sajaem@narod.ru
//  Если у Вас возникли технические вопросы, напишите нам: shop@iarduino.ru

#ifndef iarduino_I2C_IO_h
#define iarduino_I2C_IO_h

#if defined(ARDUINO) && (ARDUINO >= 100)
#include <Arduino.h>
#else
#include <WProgram.h>
#endif

#define	EXP_REG_INPUT	0														//	Адрес регистра порта входа						Запись в этот регистр игнорируется. Он отображает фактическое состояние логических уровней на входах, независимо от того как сконфигурированы выводы (вход/выхо) в регистре конфигурации выводов.
#define	EXP_REG_OUTPUT	1														//	Адрес резистра порта выхода						Устанавливает логический уровень на выходе, если он был сконфигурирован как выход в регистре конфигурации выводов. Если вывод сконфигурирован как вход, то данные этого регистра на него не влияют. Чтение значений из этого регистра вернёт установленное, а не фактическое состояние входа.
#define	EXP_REG_INVERT	2														//	Адрес регистра инверсии входов					Установленные биты регистра заставляют читать инверсное состояние входов.
#define	EXP_REG_CONFIG	3														//	Адрес регистра конфигурации выводов				Устанавливает направление работы выводов. 1-вход с высоким импендансом (по умолчанию), 0-выход.

class iarduino_I2C_IO{
	public:		iarduino_I2C_IO			(uint8_t=0xFF    );						//	Объявление	экземпляра класса					([адрес модуля]        )
	/**	пользовательские функции **/
		void	begin					(void);									//	
		void	pinMode					(uint8_t, uint8_t);						//	Установка	режима работы вывода				(№ вывода, режим работы)
		bool	digitalRead				(uint8_t         );						//	Чтение		логического состояния на входе		(№ вывода              )
		void	digitalWrite			(uint8_t, bool   );						//	Установка	логического состояния на выходе		(№ вывода, состояние   )

		void	SetModeRegister			(uint8_t         );						//	Установка	режима работы всех выводов модуля	(байт режимов работы   )
		void	SetOutputRegister		(uint8_t         );						//	Установка	состояний на всех выходах модуля	(№ вывода, состояние   )
		uint8_t	GetInputRegister		(void            );						//	Чтение		состояний со всех входов модуля		(без параметров        )
		
	private:
	/**	внутренние переменные **/
		uint8_t	EXP_uint_ADDRESS_OBJ  =	 0xFF;									//	адрес		модуля на шине I2C					(указанный при создании объекта)
		uint8_t	EXP_uint_ADDRESS      =	 0x20;									//	адрес		модуля на шине I2C					(реальный)
		uint8_t	EXP_uint_BIT          =	 0;										//	позиция		бита в модуле						(7-0)
		uint8_t	EXP_uint_COMMAND      =	 0;										//	команда		состоящая из адреса регистра		(0-3)
		uint8_t	EXP_uint_DATA         =	 0;										//	данные		для передачи по шине I2C
	   uint16_t	I2C_mass_STATUS[0x04] =	{100,1000,1,0x78};						//	скорость	работы шины в кГц (макс F_CPU/2), ожидание сброса флага TWINT в циклах, флаг успешного результата, содержимое флагов TWS регистра состояния TWSR
	/**	внутренние функции **/
		bool	EXP_func_FIND			(uint8_t,	uint8_t);					//	Поиск		переметров для передачи				(№ вывода,	адрес регистра)
	/**	функции для работы с шиной I2C **/
		void	I2C_func_BEGIN			(void);									//	установка	скорости работы шины I2C			(без параметров)
		void	I2C_func_START			(void);									//	выполнение	состояния START						(без параметров)
		void	I2C_func_RESTART		(void);									//	выполнение	состояния RESTART					(без параметров)
		void	I2C_func_STOP			(void);									//	выполнение	состояния STOP						(без параметров)
		void	I2C_func_SEND_ID		(uint8_t, bool);						//	передача	первого байта						(ID-адрес модуля, бит RW)
		void	I2C_func_WRITE_BYTE		(uint8_t);								//	передача	одного байта						(байт для передачи)
		uint8_t	I2C_func_READ_BYTE		(bool);									//	получение	одного байта						(бит подтверждения ACK/NACK)
		
		void	I2C_func_READ			(void);									//	чтение		одного байта из регистра EXP_uint_COMMAND от модуля EXP_uint_ADDRESS в  переменную EXP_uint_DATA
		void	I2C_func_WRITE			(void);									//	запись		одного байта в  регистр  EXP_uint_COMMAND к  модулю EXP_uint_ADDRESS из переменной EXP_uint_DATA
};

#endif